'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _isPlainObject = require('is-plain-object');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _isPromise = require('is-promise');

var _isPromise2 = _interopRequireDefault(_isPromise);

var _container = require('./container');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createStore = function createStore(store, update) {
  for (var name in store) {
    var obj = store[name];
    var list = (0, _container.get)(obj);
    if (list) {
      list.push(update);
    } else {
      (0, _container.set)(obj, [update]);
    }
  }
};

var _setStore = function _setStore(store, obj) {
  var list = (0, _container.get)(store);
  if (list) {
    (0, _objectAssign2.default)(store, obj);
    list.forEach(function (update) {
      return update();
    });
  }
};

var delStore = function delStore(store, update) {
  for (var name in store) {
    var obj = store[name];
    var list = (0, _container.get)(obj);
    if (list) {
      var array = list.filter(function (item) {
        return item !== update;
      });
      if (array.length >= 0) {
        (0, _container.set)(obj, array);
      } else {
        (0, _container.del)(obj);
      }
    }
  }
};

var transformStore = function transformStore(storeMap, target) {
  var map = {};

  var _loop = function _loop(name) {
    map[name] = {};
    var store = storeMap[name];
    var event = {
      target: target,
      store: store,
      setStore: function setStore(obj) {
        return _setStore(store, obj);
      }
    };

    // default function 'set'
    store.setStore = store.setStore || function (obj) {
      return obj;
    };

    var _loop2 = function _loop2(key) {
      var item = store[key];
      map[name][key] = typeof item === 'function' ? function () {
        var obj = item.apply(undefined, Array.prototype.slice.call(arguments).concat([event]));
        if ((0, _isPromise2.default)(obj)) {
          obj.then(function (value) {
            return event.setStore(value);
          });
        } else if ((0, _isPlainObject2.default)(obj)) {
          event.setStore(obj);
        }
      } : item;
    };

    for (var key in store) {
      _loop2(key);
    }
  };

  for (var name in storeMap) {
    _loop(name);
  }
  return map;
};

exports.default = { createStore: createStore, setStore: _setStore, delStore: delStore, transformStore: transformStore };
module.exports = exports['default'];